@@ -28,7 +28,6 @@
 #include "./mac-internal.h"
 #include "bufhelp.h"
 #include "cipher.h"
-#include "hmac256.h"
 
 
 static int
@@ -241,6 +240,11 @@ check_one (int algo,
   const unsigned char *digest;
 
 /*   printf ("HMAC algo %d\n", algo); */
+
+  /* Skip test with shoter key in FIPS mode.  */
+  if (fips_mode () && keylen < 14)
+    return NULL;
+
   if (trunc)
     {
       if (_gcry_md_get_algo_dlen (algo) < expectlen)
@@ -559,10 +563,6 @@ selftests_sha256 (int extended, selftest_report_func_t report)
 
   for (tvidx=0; tv[tvidx].desc; tvidx++)
     {
-      hmac256_context_t hmachd;
-      const unsigned char *digest;
-      size_t dlen;
-
       what = tv[tvidx].desc;
       errtxt = check_one (GCRY_MD_SHA256,
                           tv[tvidx].data, strlen (tv[tvidx].data),
@@ -570,30 +570,6 @@ selftests_sha256 (int extended, selftest_report_func_t report)
                           tv[tvidx].expect, DIM (tv[tvidx].expect), 0);
       if (errtxt)
         goto failed;
-
-      hmachd = _gcry_hmac256_new (tv[tvidx].key, strlen (tv[tvidx].key));
-      if (!hmachd)
-        {
-          errtxt = "_gcry_hmac256_new failed";
-          goto failed;
-        }
-      _gcry_hmac256_update (hmachd, tv[tvidx].data, strlen (tv[tvidx].data));
-      digest = _gcry_hmac256_finalize (hmachd, &dlen);
-      if (!digest)
-        {
-          errtxt = "_gcry_hmac256_finalize failed";
-          _gcry_hmac256_release (hmachd);
-          goto failed;
-        }
-      if (dlen != DIM (tv[tvidx].expect)
-          || memcmp (digest, tv[tvidx].expect, DIM (tv[tvidx].expect)))
-        {
-          errtxt = "does not match in second implementation";
-          _gcry_hmac256_release (hmachd);
-          goto failed;
-        }
-      _gcry_hmac256_release (hmachd);
-
       if (!extended)
         break;
     }
@@ -1337,158 +1313,158 @@ static const gcry_mac_spec_ops_t hmac_ops = {
 
 
 #if USE_SHA1
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha1 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha1 = {
   GCRY_MAC_HMAC_SHA1, {0, 1}, "HMAC_SHA1",
   &hmac_ops
 };
 #endif
 #if USE_SHA256
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha256 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha256 = {
   GCRY_MAC_HMAC_SHA256, {0, 1}, "HMAC_SHA256",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha224 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha224 = {
   GCRY_MAC_HMAC_SHA224, {0, 1}, "HMAC_SHA224",
   &hmac_ops
 };
 #endif
 #if USE_SHA512
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha512 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha512 = {
   GCRY_MAC_HMAC_SHA512, {0, 1}, "HMAC_SHA512",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha384 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha384 = {
   GCRY_MAC_HMAC_SHA384, {0, 1}, "HMAC_SHA384",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha512_256 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha512_256 = {
   GCRY_MAC_HMAC_SHA512_256, {0, 1}, "HMAC_SHA512_256",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha512_224 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha512_224 = {
   GCRY_MAC_HMAC_SHA512_224, {0, 1}, "HMAC_SHA512_224",
   &hmac_ops
 };
 
 #endif
 #if USE_SHA3
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_224 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_224 = {
   GCRY_MAC_HMAC_SHA3_224, {0, 1}, "HMAC_SHA3_224",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_256 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_256 = {
   GCRY_MAC_HMAC_SHA3_256, {0, 1}, "HMAC_SHA3_256",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_384 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_384 = {
   GCRY_MAC_HMAC_SHA3_384, {0, 1}, "HMAC_SHA3_384",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_512 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sha3_512 = {
   GCRY_MAC_HMAC_SHA3_512, {0, 1}, "HMAC_SHA3_512",
   &hmac_ops
 };
 #endif
-#ifdef USE_GOST_R_3411_94
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_gost3411_94 = {
+#if USE_GOST_R_3411_94
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_gost3411_94 = {
   GCRY_MAC_HMAC_GOSTR3411_94, {0, 0}, "HMAC_GOSTR3411_94",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_gost3411_cp = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_gost3411_cp = {
   GCRY_MAC_HMAC_GOSTR3411_CP, {0, 0}, "HMAC_GOSTR3411_CP",
   &hmac_ops
 };
 #endif
-#ifdef USE_GOST_R_3411_12
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_stribog256 = {
+#if USE_GOST_R_3411_12
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_stribog256 = {
   GCRY_MAC_HMAC_STRIBOG256, {0, 0}, "HMAC_STRIBOG256",
   &hmac_ops
 };
 
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_stribog512 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_stribog512 = {
   GCRY_MAC_HMAC_STRIBOG512, {0, 0}, "HMAC_STRIBOG512",
   &hmac_ops
 };
 #endif
 #if USE_WHIRLPOOL
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_whirlpool = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_whirlpool = {
   GCRY_MAC_HMAC_WHIRLPOOL, {0, 0}, "HMAC_WHIRLPOOL",
   &hmac_ops
 };
 #endif
 #if USE_RMD160
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_rmd160 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_rmd160 = {
   GCRY_MAC_HMAC_RMD160, {0, 0}, "HMAC_RIPEMD160",
   &hmac_ops
 };
 #endif
 #if USE_TIGER
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_tiger1 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_tiger1 = {
   GCRY_MAC_HMAC_TIGER1, {0, 0}, "HMAC_TIGER",
   &hmac_ops
 };
 #endif
 #if USE_MD5
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_md5 = {
-  GCRY_MAC_HMAC_MD5, {0, 0}, "HMAC_MD5",
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_md5 = {
+  GCRY_MAC_HMAC_MD5, {0, 1}, "HMAC_MD5",
   &hmac_ops
 };
 #endif
 #if USE_MD4
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_md4 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_md4 = {
   GCRY_MAC_HMAC_MD4, {0, 0}, "HMAC_MD4",
   &hmac_ops
 };
 #endif
 #if USE_MD2
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_md2 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_md2 = {
   GCRY_MAC_HMAC_MD2, {0, 0}, "HMAC_MD2",
   &hmac_ops
 };
 #endif
 #if USE_BLAKE2
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_512 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_512 = {
   GCRY_MAC_HMAC_BLAKE2B_512, {0, 0}, "HMAC_BLAKE2B_512",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_384 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_384 = {
   GCRY_MAC_HMAC_BLAKE2B_384, {0, 0}, "HMAC_BLAKE2B_384",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_256 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_256 = {
   GCRY_MAC_HMAC_BLAKE2B_256, {0, 0}, "HMAC_BLAKE2B_256",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_160 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2b_160 = {
   GCRY_MAC_HMAC_BLAKE2B_160, {0, 0}, "HMAC_BLAKE2B_160",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_256 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_256 = {
   GCRY_MAC_HMAC_BLAKE2S_256, {0, 0}, "HMAC_BLAKE2S_256",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_224 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_224 = {
   GCRY_MAC_HMAC_BLAKE2S_224, {0, 0}, "HMAC_BLAKE2S_224",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_160 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_160 = {
   GCRY_MAC_HMAC_BLAKE2S_160, {0, 0}, "HMAC_BLAKE2S_160",
   &hmac_ops
 };
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_128 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_blake2s_128 = {
   GCRY_MAC_HMAC_BLAKE2S_128, {0, 0}, "HMAC_BLAKE2S_128",
   &hmac_ops
 };
 #endif
 #if USE_SM3
-gcry_mac_spec_t _gcry_mac_type_spec_hmac_sm3 = {
+const gcry_mac_spec_t _gcry_mac_type_spec_hmac_sm3 = {
   GCRY_MAC_HMAC_SM3, {0, 0}, "HMAC_SM3",
   &hmac_ops
 };
